/* 
 * Copyright (C) 2010, 2011 johnc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "communicate.h"

// HTTP socket connect
static int
ht_connect(const char *domain, int port)
{
	int sock;
   	struct hostent *site;
   	struct sockaddr_in addr;
   	site = gethostbyname(domain);
   	if(!site)
	   	return -2;

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if(sock < 0)
		return -1;
	memset(&addr, 0, sizeof(struct sockaddr_in));
	memcpy(&addr.sin_addr, site->h_addr_list[0], site->h_length);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);

	if(connect(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0)
		sock = 0;
	return sock;
}

// HTTP character encode
// FIXME: more encode character needed??
static void
ht_encode(const char *frm, char *to, int length)
{
	int fi, ti;
	fi = ti = 0;
	for(fi = 0; fi < length; ++fi)
	{
		if(isspace(frm[fi])){
			to[ti++] = '+';
		}else if(frm[fi] == '<'){
			to[ti++] = '%';
			to[ti++] = '3';
			to[ti++] = 'C';
		}else if(frm[fi] == '>'){
			to[ti++] = '%';
			to[ti++] = '3';
			to[ti++] = 'E';
		}else if(frm[fi] == '.'){
			to[ti++] = '%';
			to[ti++] = '2';
			to[ti++] = 'E';
		}else{
			to[ti++] = frm[fi];
		}
	}
	to[ti] = '\0';
}

// google translate socket
int
gt_socket()
{
	return ht_connect("translate.google.com", 80);
}

// FRM is the string to be tranlsated
int
gt_send(int sock, const char *frm)
{
	char header[SIZE_LIMIT * 2] = "\0";	
	char content[SIZE_LIMIT] = "\0";
	int length;
	char *encode;
	length = strlen(frm);
	encode = (char *)malloc(sizeof(char) * length * 2 + 1);
	ht_encode(frm, encode, length);

	strcat(content, "langpair=en%7Czh-CN&text=");
	strcat(content, encode);
	strcat(content, "&ie=UTF-8&hl=zh-CN");

	sprintf(header, POST_FMT, strlen(content));
	strcat(header, content);
	strcat(header, "\r\n");
	send(sock, header, strlen(header), 0);

	free(encode);
}

// AMONG_P indicates if STR is part of the translated result:
// 		if 1, paste it into RESULT
// return -1 if no more STR is needed
static int
paste_result(char *result, char *str, int among_p)
{
	char *beg, *end;
	static int count = 0;
	beg = strstr(str, " title=");
	if(beg){
		++count;
		// FIXME: this is definitely not perfect, but it works anyway
		if(count != 2){
			return among_p;
		}
	}
	if(among_p){
		end = strstr(str, "</span>");
		if(end){
			*end = '\0';
			// no more STR is needed
			among_p = -1;
		}
		strcat(result, str);
	}else if(beg){
		end = strstr(beg, "</span>");
		beg = strstr(beg, ">") + 1;
		if(end){
			*end = '\0';
			among_p = -1;
		}else{
			among_p = 1;
		}
		strcat(result, beg);
	}
	return among_p;
}

// receive translated RESULT from socket
void
gt_recv(int sock, char *result)
{
	char segment[1025];
	int among_p = 0;
	while((recv(sock, segment, 1024, 0)) > 0){

		if(among_p == -1)
			break;

		among_p = paste_result(result, segment, among_p);
	}
}

// I am doing this just for fun...
inline void
gt_close(int sock)
{
	close(sock);
}
